<?php
/*
 * Gallery - a web based photo album viewer and editor
 * Copyright (C) 2000-2008 Bharat Mediratta
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street - Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * This view creates and sends the zip file.
 * @package ZipCart
 * @author Alan Harder <alan.harder@sun.com>
 * @version $Revision: 17580 $
 */

include_once("ZipFile.php");

class DownloadView extends GalleryView {

    /**
     * @see GalleryView::isImmediate
     */
    function isImmediate() {
	return true;
    }

    /**
     * @see GalleryView::renderImmediate
     */
    function renderImmediate($status, $error) {
	global $gallery;
	$platform =& $gallery->getPlatform();
	$phpVm = $gallery->getPhpVm();

	$Session =& $gallery->getSession(); 
	
	if ($Session->exists("ZipCartContents")) {
	    $size = 0;
	    /**
	     * Try to prevent Apache's mod_deflate from gzipping the output a 2nd time since
	     * broken versions of mod_deflate sometimes get the byte count wrong.
	     */
	    @$phpVm->ini_set('zlib.output_compression', 0);
	    if (function_exists('apache_setenv') && !@$gallery->getConfig('apacheSetenvBroken')) {
		@apache_setenv('no-gzip', '1');
	    }

	    /* preg i modifier is affected by locale */
	    $sendMultipart = !preg_match('{[Mm][Ss][Ii][Ee] |[Ss][Aa][Ff][Aa][Rr][Ii]}',
					 GalleryUtilities::getServerVar('HTTP_USER_AGENT'));
	    if ($sendMultipart) {
		/* Send multipart reply: zip + html-to-reload */
		$phpVm->header('Content-Type: multipart/mixed; boundary=G2ZipCart');
		print "--G2ZipCart\nContent-Type: application/zip\n";
		print "Content-Disposition: inline; filename=\"G2cart.zip\"\n";

		if ($size > 0) {
		    print "Content-Length: " . $size . "\n";
		}
		print "\n";
	    } else {
		/* Poor IE6 doesn't know multipart.. just send zip */
		/* Safari doesn't get filename in multipart and may even crash */
		$phpVm->header('Content-Type: application/zip');
		$phpVm->header('Content-Disposition: inline; filename="G2cart.zip"');
		if ($size > 0) {
		    $phpVm->header("Content-Length: $size");
		}
	    }

	    // Send the contents of the zip file
	    $cartItemIds = unserialize(base64_decode($Session->get("ZipCartContents")));
	    $MyZipFile = new zipfile();
	
	    $itemIds = array_keys($cartItemIds);
	    list ($ret, $items) = GalleryCoreApi::loadEntitiesById($itemIds, 'GalleryDataItem');
	    if ($ret) {
		return array($ret, null);
	    }
	    list ($ret, $preferreds) = GalleryCoreApi::fetchPreferredsByItemIds($itemIds);
	    if ($ret) {
		return array($ret, null);
	    }
	    list ($ret, $resizes) = GalleryCoreApi::fetchResizesByItemIds($itemIds);
	    if ($ret) {
		return array($ret, null);
	    }
	    list ($ret, $thumbnails) = GalleryCoreApi::fetchThumbnailsByItemIds($itemIds);
	    if ($ret) {
		return array($ret, null);
	    }
	    list ($ret, $permissions) = GalleryCoreApi::fetchPermissionsForItems($itemIds);
	    if ($ret) {
		return array($ret, null);
	    }

	    $albumDir = $gallery->getConfig('data.gallery.albums');
	    $i = 0;
	    
	    foreach ($items as $item) {
		if ($i++ % 50) {
		    $gallery->guaranteeTimeLimit(30);
		}
		list ($ret, $path) = $item->fetchPath();
		if ($ret) {
		    return array($ret, null);
		}
		$relativePath = (strpos($path, $albumDir) === 0) ?
				substr($path, strlen($albumDir)) : basename($path);

		$itemId = $item->getId();
		$itemMimeType = $item->getMimeType();

		if (isset($permissions[$itemId]['core.viewSource'])) {
		    if (isset($preferreds[$itemId])) {
			$item = $preferreds[$itemId];
		    }
		} else if (isset($permissions[$itemId]['core.viewResizes'])
			&& !empty($resizes[$itemId])) {
		    $maxSize = null;
		    foreach ($resizes[$itemId] as $resize) {
			$size = $resize->getDerivativeSize();
			if (!isset($maxSize) || $size > $maxSize) {
			    $item = $resize;
			    $maxSize = $size;
			}
		    }
		} else if (isset($thumbnails[$itemId])) {
		    $item = $thumbnails[$itemId];
		} else {
		    $item = null;
		}

		if (!isset($item)) {
		    /* Nothing to download for this item! */
		    continue;
		}

		if (GalleryUtilities::isA($item, 'GalleryDerivative')) {
		    list ($ret, $item) =
			GalleryCoreApi::rebuildDerivativeCacheIfNotCurrent($item->getId());
		    if ($ret) {
			return array($ret, null);
		    }
		    /* Bug 1462984: Make sure we use the right extension */
		    $derivativeMimeType = $item->getMimeType();
		    if ($derivativeMimeType != $itemMimeType) {
			list ($ret, $mimeExtensions) =
			    GalleryCoreApi::convertMimeToExtensions($derivativeMimeType);
			if ($ret) {
			    return array($ret, null);
			}
			if (!empty($mimeExtensions)) {
			    $base = GalleryUtilities::getFileBase($relativePath);
			    $relativePath = $base . '.' . $mimeExtensions[0];
			}
		    }
		}

		list ($ret, $path) = $item->fetchPath();
		if ($ret) {
		    return array($ret, null);
		}

		/* Copy file into zip file with right subdir/filename */
		$target = $relativePath;
		$MyZipFile->add_dir(dirname($target) . "/");
		$MyZipFile->add_file($path, $target);
	    }

	    $gallery->guaranteeTimeLimit(60);

	    $MyZipFile->finish();

	    $Session->remove("ZipCartContents");
	    // contents sent

	    if ($sendMultipart) {
		print "\n--G2ZipCart\nContent-Type: text/html\nPragma: No-cache\n\n";
		print "<html><body onload=\"location.reload()\"></body></html>\n\n";
		print "--G2ZipCart--\n";
	    }
	} else {
	    /* On reload return to View Cart */
	    $urlGenerator =& $gallery->getUrlGenerator();
	    $phpVm->header('Location: ' . $urlGenerator->generateUrl(
		array('view' => 'cart.ViewCart'), array('htmlEntities' => false)));
	}

	return null;
    }
}
?>
